#include "xml_parser.h"

void xml_parser::Node_traverse(const QDomNode &p_dNode, QStandardItem *p_item_ptr)
{
    QDomNode Dom_node = p_dNode.firstChild();
    while(!Dom_node.isNull())
    {
        if(Dom_node.isElement())
        {
            QDomElement Dom_elem = Dom_node.toElement();
            if(!Dom_elem.isNull())
            {
                if(Dom_elem.tagName() == "array")
                {
                    QStandardItem *item_array = new QStandardItem("OrgName");
                    QStandardItem *item_text = new QStandardItem();
                    FILE_tech_Item = item_text;
                    p_item_ptr->appendRow({item_array, item_text});
                    Node_traverse(Dom_node, item_array);

                }
                else if(Dom_elem.tagName() == "geoData")
                {
                    QStandardItem *item_geoData = new QStandardItem(Dom_elem.tagName());
                    p_item_ptr->appendRow(item_geoData);
                    Node_traverse(Dom_node, item_geoData);

                }
                else if (Dom_elem.tagName() == "Name")
                {
                     FILE_tech_Item->setText(Dom_elem.text());
                     Node_traverse(Dom_node, p_item_ptr);
                }
                else
                {
                    QStandardItem *item_tagName = new QStandardItem(Dom_elem.tagName());
                    QStandardItem *item_text = new QStandardItem(Dom_elem.text());
                    p_item_ptr->appendRow({nullptr, nullptr, item_tagName, item_text});
                    Node_traverse(Dom_node, p_item_ptr);

                }

            }
        }
        Dom_node = Dom_node.nextSibling();
    }
}

void xml_parser::Simple_node_traverse(const QDomNode &p_dNode, QStandardItem *p_item_ptr)
{
    QDomNode Dom_node = p_dNode.firstChild();
    while(!Dom_node.isNull())
    {
        if(Dom_node.isElement())
        {
            QDomElement domElement = Dom_node.toElement();

            if (!domElement.isNull())
            {
                qDebug() << domElement.tagName();
                qDebug() << domElement.text();
                QStandardItem *item_tag = new QStandardItem(domElement.tagName());
                QStandardItem *item_text = new QStandardItem(domElement.text());
                p_item_ptr->appendRow({item_tag, item_text});
                Simple_node_traverse(Dom_node, p_item_ptr);
            }

        }
        Dom_node = Dom_node.nextSibling();
    }
}

xml_parser::xml_parser(QWidget* tmp_parent): QTreeView(tmp_parent)
{
    this->setEditTriggers(QTreeView::NoEditTriggers);
    FILE_std_Item_Model = new QStandardItemModel(this);
    FILE_std_Item_Model->setColumnCount(4);
    FILE_std_Item_Model->setHorizontalHeaderLabels(QStringList() << "Files" << "Organization Name" << "Attributes" << "Values");
    FILE_parent_Item = FILE_std_Item_Model->invisibleRootItem();

    MENU_context_Root_Menu = new QMenu(this);

    MENU_context_Root_Menu->addAction("Activate");
    MENU_context_Root_Menu->addAction("Close");

    this->setContextMenuPolicy(Qt::CustomContextMenu);



    // connects

    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(Context_menu_request_slot(QPoint)));
    connect(MENU_context_Root_Menu, SIGNAL(triggered(QAction*)), this, SLOT(Context_menu_reser_slot(QAction*)));
}

void xml_parser::Load_slot()
{
    QString open_xml_file_directory = QFileDialog::getOpenFileName();
       if (FILE_directory_List.contains(open_xml_file_directory))
           return;

       qDebug() << "choosen file directory: " << open_xml_file_directory;
       FILE_directory_List << open_xml_file_directory;

       //xml parsing(into itself)

       QDomDocument Dom_document;
       FILE_current_File = new QFile(open_xml_file_directory);
       QStringList list = open_xml_file_directory.split("/");
       QString fName = list.last();
       qDebug() << "filename: " << fName;

       if (FILE_current_File->open(QIODevice::ReadOnly))
       {
           if (Dom_document.setContent(FILE_current_File))
           {
               QDomElement domElem = Dom_document.documentElement();
               if(domElem.isNull())
                   qDebug() << "bad request";
               QStandardItem *item = new QStandardItem(fName);
               FILE_root_List << item;
               FILE_parent_Item->appendRow(item);

               Node_traverse(domElem, item);

               Set_current_root_slot(FILE_std_Item_Model->indexFromItem(item));
           }
           else
           {
               qDebug() << "Something goes wrong";
           }
           FILE_current_File->close();
       }
       else
       {
           qDebug() << "Unable to open file" << open_xml_file_directory;
       }

       setModel(FILE_std_Item_Model);

}

void xml_parser::Load_slot(const QString &f_directory)
{
    FILE_directory_List << f_directory;


       QDomDocument Dom_document;
       FILE_current_File = new QFile(f_directory);
       QStringList list = f_directory.split("/");
       QString fName = list.last();

       if (FILE_current_File->open(QIODevice::ReadOnly))
       {
           if (Dom_document.setContent(FILE_current_File))
           {
               QDomElement Dom_elem = Dom_document.documentElement();
               QStandardItem *item = new QStandardItem(fName);
               FILE_root_List << item;
               FILE_parent_Item->appendRow(item);

               Node_traverse(Dom_elem, item);

               Set_current_root_slot(FILE_std_Item_Model->indexFromItem(item));
           }
           FILE_current_File->close();
       }

       setModel(FILE_std_Item_Model);

}

void xml_parser::Simple_load_slot(const QString &f_directory)
{


    FILE_directory_List << f_directory;

    QDomDocument Dom_document;
    FILE_current_File = new QFile(f_directory);
    QStringList list = f_directory.split("/");
    QString fName = list.last();

    if (FILE_current_File->open(QIODevice::ReadOnly))
    {
        if (Dom_document.setContent(FILE_current_File))
        {
            QDomElement Dom_elem = Dom_document.documentElement();
            QStandardItem *item = new QStandardItem(fName);
            FILE_root_List << item;
            if(FILE_parent_Item->rowCount() != 0)
                FILE_parent_Item->removeRow(0);
            FILE_parent_Item->appendRow(item);

            Simple_node_traverse(Dom_elem, item);

        }
        FILE_current_File->close();
    }

    setModel(FILE_std_Item_Model);

}

void xml_parser::Drop_all_slot()
{
    for(auto root : FILE_root_List)
    {
        Drop_single_item_slot(FILE_std_Item_Model->indexFromItem(root));
    }
    FILE_root_List.clear();
    FILE_directory_List.clear();
    emit(Send_file(nullptr));
}

QList<QStandardItem *> *xml_parser::Get_root_list_slot()
{
    return &FILE_root_List;
}

void xml_parser::Set_current_root_slot()
{
    QString root = qobject_cast <QAction*>(sender())->text();
    auto tmp_iter = FILE_root_List.begin();
    int counter = 0;
    while (tmp_iter!= FILE_root_List.end())
    {
       if(tmp_iter.i->t()->text() == root)
        {
           scrollTo(FILE_std_Item_Model->indexFromItem(tmp_iter.i->t()));
           tmp_iter.i->t()->setFont(QFont("Times New Roman,12,-1,5,50,0,0,0,0,0", 9, QFont::Bold));
           FILE_current_File = new QFile(FILE_directory_List.at(counter));
           emit Send_file(FILE_current_File);
        }
        else
        {
           tmp_iter.i->t()->setFont(QFont ("Times New Roman,12,-1,5,50,0,0,0,0,0", 9));
           qDebug() << tmp_iter.i->t()->text();
        }
            counter++;
            tmp_iter++;
        }

}

void xml_parser::Set_current_root_slot(QModelIndex inx)
{
    int pos = 0;
    for(auto root : FILE_root_List)
    {
        if(inx == FILE_std_Item_Model->indexFromItem(root))
        {
            scrollTo(FILE_std_Item_Model->indexFromItem(root));
            root->setFont(QFont("Times New Roman,12,-1,5,50,0,0,0,0,0", 9, QFont::Bold));
            FILE_current_File = new QFile(FILE_directory_List.at(pos));
            emit Send_file(FILE_current_File);
        }
        else
        {
            root->setFont(QFont("Times New Roman,12,-1,5,50,0,0,0,0,0", 9, QFont::Bold));
            qDebug() << root->text();
        }
        ++pos;
    }

}

void xml_parser::Context_menu_request_slot(QPoint position)
{
    bool is_Root = false;
    for(auto elem : FILE_root_List)
    {
        qDebug() << FILE_std_Item_Model->data(this->indexAt(position)).toString();
        qDebug() << this->indexAt(position) << " == " << FILE_std_Item_Model->indexFromItem(elem);
        if(this->indexAt(position) == FILE_std_Item_Model->indexFromItem(elem))
        is_Root = true;

    }
    if(is_Root)
    {
        FILE_last_Context_Request = this->indexAt(position);
        MENU_context_Root_Menu->exec(mapToGlobal(position));
    }
}

void xml_parser::Context_menu_reser_slot(QAction * action)
{
    qDebug() << action->text() << MENU_context_Root_Menu->actions().indexOf(action);
      if (MENU_context_Root_Menu->actions().indexOf(action) == 0)
      {
          Set_current_root_slot(FILE_last_Context_Request);
      }
      else
      {
          Drop_single_item_slot(FILE_last_Context_Request);
      }

}

void xml_parser::Drop_single_item_slot(QModelIndex inx )
{
    FILE_std_Item_Model->invisibleRootItem()->removeRow(inx.row());
    FILE_directory_List.removeAt(inx.row());
    FILE_root_List.removeAt(inx.row());

}
